﻿using System;
using System.Drawing;
using System.Runtime.InteropServices;
using System.Windows.Forms;
using Steema.TeeChart;
using Steema.TeeChart.Editors.Series;
using Steema.TeeChart.Styles;

namespace TestingTeeChartScaling
{
    public partial class Form1 : Form
    {
        private readonly double[] m_RectangleX = new double[5];
        private readonly double[] m_RectangleY = new double[5];
        private readonly Line m_Line = new Line();

        private static short HORZSIZE = 4;     // Horizontal size in millimeters 
        private static short VERTSIZE = 6;     // Vertical size in millimeters 


        [DllImport("gdi32.dll")]
        private static extern Int32 GetDeviceCaps(IntPtr hdc, Int32 capindex);

        public Form1()
        {
            InitializeComponent();

            m_Chart.Aspect.View3D = false;
            m_Chart.Aspect.SmoothingMode = System.Drawing.Drawing2D.SmoothingMode.AntiAlias;
            m_Chart.Legend.Visible = false;
            m_Chart.Location = new Point(0, 0);
            m_Chart.Name = "Testing TChart";
            m_Chart.TabIndex = 0;
            m_Chart.Chart.Header.Text = "Testing TChart";
            m_Chart.Series.Clear();
            m_Chart.Tools.Clear();

            AddSeriesToChart();

        }

        private void AddSeriesToChart()
        {
            m_RectangleX[0] = -50.0;
            m_RectangleY[0] = -50.0;

            m_RectangleX[1] = 50.0;
            m_RectangleY[1] = -50.0;

            m_RectangleX[2] = 50.0;
            m_RectangleY[2] = 50.0;

            m_RectangleX[3] = -50.0;
            m_RectangleY[3] = 50.0;

            m_RectangleX[4] = -50.0;
            m_RectangleY[4] = -50.0;

            m_Line.XValues.Count = 5;
            m_Line.XValues.Value = m_RectangleX;

            m_Line.YValues.Count = 5;
            m_Line.YValues.Value = m_RectangleY;

            m_Chart.Series.Add(m_Line);

            m_Chart.Axes.Bottom.Automatic = false;
            m_Chart.Axes.Bottom.Minimum = -100.0;
            m_Chart.Axes.Bottom.Maximum = 100.0;
            m_Chart.Axes.Bottom.Visible = false;
            m_Chart.Axes.Bottom.Ticks.Length = 0;

            m_Chart.Axes.Left.Automatic = false;
            m_Chart.Axes.Left.Minimum = -100.0;
            m_Chart.Axes.Left.Maximum = 100.0;
            m_Chart.Axes.Left.Visible = false;
            m_Chart.Axes.Left.Ticks.Length = 0;

            MakeIsometric(m_Chart);

            m_Chart.ClientSizeChanged += ChartOnClientSizeChanged;

        }

        private void ChartOnClientSizeChanged(object sender, EventArgs eventArgs)
        {
            MakeIsometric(m_Chart);
        }


        private void MakeIsometric(TChart chart)
        {
            if (chart.Width == 0 || chart.Height == 0) return;

            var axesBottomRange = chart.Axes.Bottom.Maximum - chart.Axes.Bottom.Minimum;

            var axesLeftRange = chart.Axes.Left.Maximum - chart.Axes.Left.Minimum;

            var normalizedAxesBottomRange = axesBottomRange / Convert.ToDouble(chart.Width);

            var normalizedAxesLeftRange = axesLeftRange / Convert.ToDouble(chart.Height);

            var screen = Screen.FromControl(this);

            var graphics = Graphics.FromHwnd(Handle);

            var handleOfDeviceContext = graphics.GetHdc();

            var xyscreen = Convert.ToDouble(GetDeviceCaps(handleOfDeviceContext, HORZSIZE) * screen.Bounds.Height) /
                       Convert.ToDouble(GetDeviceCaps(handleOfDeviceContext, VERTSIZE) * screen.Bounds.Width);

            normalizedAxesLeftRange = normalizedAxesLeftRange * xyscreen;

            double offset = 0.0;

            if (Math.Abs(normalizedAxesBottomRange) < 0.0 || Math.Abs(normalizedAxesLeftRange) < 0.0) return;

            if (normalizedAxesBottomRange > normalizedAxesLeftRange)
            {
                offset = 0.5 * (axesLeftRange * normalizedAxesBottomRange / normalizedAxesLeftRange - axesLeftRange);

                chart.Axes.Left.SetMinMax(chart.Axes.Left.Minimum - offset, chart.Axes.Left.Maximum + offset);

                chart.Axes.Bottom.SetMinMax(chart.Axes.Bottom.Minimum + offset, chart.Axes.Bottom.Maximum - offset);
            }

            if (normalizedAxesLeftRange > normalizedAxesBottomRange)
            {
                offset = 0.5 * (axesBottomRange * normalizedAxesLeftRange / normalizedAxesBottomRange - axesBottomRange);

                chart.Axes.Bottom.SetMinMax(chart.Axes.Bottom.Minimum - offset, chart.Axes.Bottom.Maximum + offset);

                chart.Axes.Left.SetMinMax(chart.Axes.Left.Minimum + offset, chart.Axes.Left.Maximum - offset);
            }

            //m_Chart.Header.Text = "Offset: " + offset +
            //                      "\n Vertical Minimum: " + chart.Axes.Left.Minimum +
            //                      " Vertical Maximum: " + chart.Axes.Left.Maximum +
            //                      "\n Horizontal Minimum: " + chart.Axes.Bottom.Minimum +
            //                      " Horizontal Maximum: " + chart.Axes.Bottom.Maximum +
            //                      " Horz csps: " + GetDeviceCaps(handleOfDeviceContext, HORZSIZE) + "Vert caps: " + GetDeviceCaps(handleOfDeviceContext, VERTSIZE);

            //graphics.ReleaseHdc(handleOfDeviceContext);

            //graphics.Dispose();

        }
    }
}
